/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.storage;

import mir.entity.EntityList;
import mir.log.LoggerWrapper;
import mir.storage.Database;
import mir.storage.DatabaseFailure;
import mircoders.entity.EntityContent;
import mircoders.entity.EntityTopics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * <b>This class implements the 1-n-relation between
 * content and topic
 *
 */

public class DatabaseContentToTopics extends Database {

  private static DatabaseContentToTopics instance;

  public synchronized static DatabaseContentToTopics getInstance() {
    if (instance == null) {
      instance = new DatabaseContentToTopics();
    }
    return instance;
  }

  private DatabaseContentToTopics() {
    super();

    logger = new LoggerWrapper("Database.ContentToTopics");
    mainTable="content_x_topic";
    entityClass = mir.entity.GenericEntity.class;
  }

  /**
   * This class return an EntityList of Topics
   */
  public EntityList getTopics(EntityContent content) {
    EntityList returnList=null;
    if (content != null) {

      String id = content.getId();
      try {
        ArrayList extraTables = new ArrayList();
        extraTables.add(mainTable+" cxt");
        returnList = DatabaseTopics.getInstance()
      	               .selectByWhereClauseWithExtraTables("t",extraTables,
			                       "t.id=cxt.topic_id and  cxt.content_id="+id );
      }
      catch (Exception e) {
        logger.error("-- get topics failed " + e.toString());
      }
    }
    return returnList;
  }

  /**
   * Returns a List of String-Objects from a content-id.
   */
  public List getTopicsOfContent(String contentId)
    throws DatabaseFailure {
    ArrayList returnList = new ArrayList();

    if (contentId != null) {
      String sql = "select topic_id from " + mainTable + " where content_id=" + contentId;
      Connection con=null;Statement stmt=null;
      try {
        con = obtainConnection();

        // should be a preparedStatement because is faster
        stmt = con.createStatement();
        ResultSet rs = executeSql(stmt,sql);
        if(rs!=null){
          while(rs.next()){
            returnList.add(Integer.toString(rs.getInt("topic_id")));
          }
        }
      }
      catch (Exception e) {
        logger.error("DatabaseContentToTopics.getTopicsOfContent: " + e.getMessage());
      }
      finally {
        freeConnection(con,stmt);
      }
    }
    return returnList;
  }

  private String getIdListExpression(List aList) {
    String result = "";

    Iterator i = aList.iterator();

    while (i.hasNext()) {
      result = result + i.next().toString();
      if (i.hasNext())
        result = result + ", ";
    }
    return result;
  }

  public void setTopics(String anArticleId, String [] aTopics) throws DatabaseFailure {
    if (aTopics==null)
      setTopics(anArticleId, (List) null);
    else
      setTopics(anArticleId, Arrays.asList(aTopics));
  }

  public void setTopics(String anArticleId, List aTopics) throws DatabaseFailure {
    List newTopics = new ArrayList();
    if (aTopics!=null) {
      Iterator i = aTopics.iterator();

      while (i.hasNext()) {
        newTopics.add(new Integer(Integer.parseInt((String) i.next())));
      }
    }

    List currentTopics = getTopicsOfContent(anArticleId);
    logger.debug("New topics = " + newTopics.toString());
    logger.debug("Current topics = " + currentTopics.toString());
    List topicsToDelete = new ArrayList(currentTopics);
    topicsToDelete.removeAll(newTopics);
    List topicsToAdd = new ArrayList(newTopics);
    topicsToAdd.removeAll(currentTopics);
    logger.debug("to delete = " + topicsToDelete.toString());
    logger.debug("to add = " + topicsToAdd.toString());


    if (!topicsToDelete.isEmpty()) {
      String sql =
          "delete from " + mainTable + " " +
          "where content_id=" + anArticleId +
          "        and topic_id in (" + getIdListExpression(topicsToDelete) + ")";

      Connection connection=null;
      Statement statement=null;
      try {
        connection = obtainConnection();
        statement = connection.createStatement();
        executeUpdate(statement, sql);
      }
      catch (Exception e) {
        logger.error("-- deleting topics failed");
      }
      finally {
        try {
          freeConnection(connection, statement);
        }
        catch (Throwable t) {
        }
      }
    }

    Iterator i = topicsToAdd.iterator();
    while (i.hasNext()) {
      Integer topicId = (Integer) i.next();
      String sql =
          "insert into " + mainTable + " (content_id, topic_id) "+
          "values (" + anArticleId + "," + topicId + ")";
      Connection connection=null;
      Statement statement=null;
      try {
        connection = obtainConnection();
        // should be a preparedStatement because is faster
        statement = connection.createStatement();
        executeUpdate(statement, sql);
      }
      catch (Exception e) {
        logger.error("-- adding topics failed");
      }
      finally {
        try {
          freeConnection(connection, statement);
        }
        catch (Throwable t) {
        }
      }
    }
  }

  public void deleteByContentId(String contentId)
    throws DatabaseFailure {
    if (contentId == null) {
      //theLog.printDebugInfo("-- delete topics failed -- no content id");
      return;
    }
    //delete all row with content_id=contentId
    String sql = "delete from "+ mainTable +" where content_id=" + contentId;

    Connection con=null;Statement stmt=null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      executeSql(stmt,sql);
    } catch (Exception e) {
      //theLog.printDebugInfo("-- delete topics failed  ");
    } finally {
      freeConnection(con,stmt);
    }
  }

  public void deleteByTopicId(String topicId)
    throws DatabaseFailure {
    if (topicId == null) {
      //theLog.printDebugInfo("-- delete topics failed -- no topic id");
      return;
    }
    //delete all row with content_id=contentId
    String sql = "delete from "+ mainTable +" where topic_id=" + topicId;

    Connection con=null;Statement stmt=null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      executeSql(stmt,sql);
    }
    catch (Exception e) {
      logger.error("-- delete topics failed ");
    }
    finally {
      freeConnection(con,stmt);
    }
  }

/**
 * Returns list of Content for a specific topic
 * @param topic
 * @return EntityList
 * @throws DatabaseFailure
 */
  public EntityList getContent(EntityTopics topic)
    throws DatabaseFailure {
    EntityList returnList=null;
    if (topic != null) {
      String id = topic.getId();
      try {
        ArrayList extraTables = new ArrayList();
        extraTables.add(mainTable+" cxt");
        returnList = DatabaseContent.getInstance()
                      .selectByWhereClauseWithExtraTables("c",extraTables,
                          "c.id=cxt.content_id and cxt.topic_id="+id );
      }
      catch (Exception e) {
        logger.error("-- get content failed");
      }
    }
    return returnList;
  }
}
